function runFEM_coeff
% Solve the equation
%     - div(A grad u) = f   in Omega,
%              u  = 0   on boundary of Omega.
% with linear P1 finite elements.

close all;

d=1;

%% Initialization
addpath(genpath(pwd));
if d==1 
    nRef = 15;
elseif d==2
    nRef = 8;
end

epsilonList = [0.5,0.1,1e-3,1e-4,1e-5]';

h = zeros(nRef,1);
err = zeros(nRef,size(epsilonList,1),1);

for k = 1:size(epsilonList,1)
    epsilon = epsilonList(k);

    %% Load triangulation
    if d==2
        c4n = load('crisscross/c4n.dat');
        n4e = load('crisscross/n4e.dat');
        n4sDb = load('crisscross/n4sDb.dat');
        n4sNb = load('crisscross/n4sNb.dat');
        nRef=8;
    elseif d==1
        %1d example
        c4n = [0;1];
        n4e = [1,2];
        n4sDb = [1;2];
        n4sNb = zeros(0,1);
        nRef=17;
    end

    for j=1:nRef
        %% refine mesh
        [c4n,n4e,n4sDb,n4sNb] = red_refine(c4n,n4e,n4sDb,n4sNb);
        % compute mesh-size
        if j==1
            if d==1
                h(j)=1/2;
            elseif d==2
                h(j) = sqrt(2)/2;
            end
        else
            h(j) = h(j-1)/2;
        end

        %% compute FEM solution
        x = FEM_coeff(c4n,n4e,n4sDb,@f, @(x) Aeps(x,epsilon));

        %% computation of L^2 error in 1d
        if d==1
            % difference at midpoints
            [c4nsorted,indxsort]=sort(c4n);
            xsorted = x(indxsort);
            errnodes = (u_exact1d(c4nsorted,epsilon)-xsorted).^2;
            err_14 = (u_exact1d(0.75*c4nsorted(1:end-1)+0.25*c4nsorted(2:end),epsilon)  ...
                - ( 0.75*xsorted(1:end-1) + 0.25* xsorted(2:end) ) ).^2;
            err_12 = (u_exact1d(0.5*c4nsorted(1:end-1)+0.5*c4nsorted(2:end),epsilon) ...
                - (0.5*xsorted(1:end-1) + 0.5 *xsorted(2:end))).^2;
            err_34 = (u_exact1d(0.25*c4nsorted(1:end-1)+0.75*c4nsorted(2:end),epsilon) ...
                - (0.25*xsorted(1:end-1) + 0.75 *xsorted(2:end))).^2;
            % Milne rule
            err(j,k) = sqrt((1/90)*h(j) ...
                *(7*sum(errnodes)+7*sum(errnodes(2:end-1)) ...
                + 32*sum(err_14) + 12*sum(err_12) + 32*sum(err_34)));
        end

        disp(['epsilon = ',num2str(epsilon),...
              '; h = ',num2str(h(j)), ...
              '; error = ',num2str(err(j,k))]);

        %% Plot solution
        if d==1
            [c4nsorted,indxsort]=sort(c4n);
            plot(c4nsorted,x(indxsort));
            hold all;
            plot(0:0.01:1,u_exact1d(0:0.01:1,epsilon));
        elseif d==2
            figure;
            trisurf(n4e,c4n(:,1),c4n(:,2),x);
        end
    end

end

%% plot convergence history plot
figure;
for k=1:size(epsilonList,1)
    loglog(h,err(:,k),'-x');
    hold all;
end
loglog(h,h.^2);
loglog(h,h);


%% save information
fid = fopen('data.txt','w');
fprintf(fid,'%s\t%s','h','h2');
for j=1:size(epsilonList,1)
    fprintf(fid,'\t%e',epsilonList(j));
end
fprintf(fid ,'\n');
dataList = [h,h.^2,err];
for k=1:size(dataList,1)
    fprintf(fid,'%e\t',dataList(k,:));
    fprintf(fid ,'\n');
end
fclose(fid);




end

%% problem input data
function val = f(x)
val = ones(size(x,1),1);
end

function val =u_exact1d(x,epsilon)
u0 = x-x.^2;
u1 = -epsilon*((1/(4*pi))*sin(2*pi*x/epsilon) ...
    - (x/(2*pi)).*sin(2*pi*x/epsilon) ...
    - (epsilon/(4*pi^2))*cos(2*pi*x/epsilon) ...
    + epsilon/(4*pi^2));
val = u0+u1;
end

function val = Aeps(x,epsilon)
val = 1/(2-cos(2*pi*x(:,1)/epsilon));
end

function val = Ahomogenized(x,~)
val = 0.5*ones(size(x,1));
end
